function Combine5(datasize, sfilename, GAP, Nr, Nc, SizeX, SizeY, contrast)
%datasize = number of samples
%normalized to SizeX*SizeY
%GAP is the space between 2 images, default is SIZE/2

if (~exist('contrast','var')),  contrast = -1;  end;

%Read in images and perform the preprocess
D = uigetdir('','Choose images directory that you want to combine to large image');
flist=dir(D);

t=cputime;

if (~exist('SizeY','var'))
    [SizeX SizeY]= FindMaxSize(flist, datasize, D);
end
% [images N names]= ReadImagesDir2(flist, datasize, SizeX, SizeY, D, contrast);

% % % if you want to normalize all images to SizeX*SizeY please use 'ReadImagesDir1'
[images N names]= ReadImagesDir1(flist, datasize, SizeX, SizeY, D); 
disp([num2str(N) ' images read in ', num2str(cputime-t),' sec']);

if (nargin<4) 
    Nc = ceil(sqrt(datasize)); Nr = ceil(datasize/Nc);
end
big_image = ones(Nr*SizeX + (Nr+1)*GAP, Nc*SizeY + (Nc+1)*GAP);
for i=1:Nr
    for j=1:Nc
        st_i=(i)*GAP+(i-1)*SizeX +1;
        en_i=(i)*GAP+(i)*SizeX ;
        st_j=(j)*GAP+(j-1)*SizeY +1;
        en_j=(j)*GAP+(j)*SizeY ;
        index=(i-1)*Nc+j;
        if (index <= N)
            big_image(st_i:en_i, st_j:en_j)=images(:,:,index);
        end
    end
end

%imshow(big_image);
disp(['Combined Image Size: ',num2str(size(big_image,1)),' x ', num2str(size(big_image,2))]);
sfilename = [sfilename '.png'];
imwrite(big_image, sfilename, 'png');
end


%% 
function [maxx maxy] = FindMaxSize(flist, datasize, D)
    fnum = length(flist);
    i = 1; j=0;
    I = cell(1,datasize);
    maxx = 0; maxy = 0;
    while ((i <= fnum)&&(j<datasize))
        if ~flist(i).isdir && ~strcmp(flist(i).name,'Thumbs.db')
            j = j + 1;
            I{j} = imread(sprintf('%s/%s',D,flist(i).name));  
            try %if I{j} is not binary
                image_bw = ~im2bw(I{j},graythresh(I{j}));
            catch
                image_bw = ~I{j};
            end
            %reduce the printing noise
            [imx imy] = size(image_bw);
            if (maxx < imx),  maxx=imx;     end;
            if (maxy < imy),  maxy=imy;     end;
        end
        i = i + 1;
    end
end
    
%% Concatenating files together, no scaling
function [images N names] = ReadImagesDir2(flist, datasize, SizeX, SizeY, D, contrast)    
    fnum = length(flist);
    i = 1;
    j = 0;
    images = ones(SizeX,SizeY,datasize,'uint8');
    I = cell(1,datasize);
    names = cell(1,datasize);
    while ((i <= fnum)&&(j<datasize)) 
        if ~flist(i).isdir && ~strcmp(flist(i).name,'Thumbs.db')
            j = j + 1;
            names{j} = flist(i).name;
            I{j} = imread(sprintf('%s/%s',D,flist(i).name));  
            try %if I{j} is not binary
                if ((contrast < 0)||(contrast>1))
                    image_bw = ~im2bw(I{j},graythresh(I{j}));
                else
                    image_bw = ~im2bw(I{j},contrast);
                end
            catch
                image_bw = ~I{j};
            end
            [imx imy] = size(image_bw);
            % % % reduce the printing noise
        %     image_d = bwmorph(image_bw,'dilate');
        %     image_m = bwmorph(image_d,'majority');
        %     image_thin = bwmorph(image_m,'thin',3);%Thinning
        %     image_bw = bwmorph(image_thin,'remove'); %Removes interior pixels
            images(1:imx,1:imy,j) = ~image_bw;
        end
        i = i + 1;
    end
    N = j;
end


%% Use when want to rescale all images to be the same size
% Resize all images to same scale => SizeX x SizeY
% Also keep the aspect ratio 
function [images N names] = ReadImagesDir1(flist, datasize, SizeX, SizeY, D)
fnum = length(flist);
i = 1;
j = 0;

images = ones(SizeX,SizeY,datasize,'uint8');
I = cell(1,datasize);
names = cell(1,datasize);
while i <= fnum    
    if ~flist(i).isdir && ~strcmp(flist(i).name,'Thumbs.db')
        j = j + 1;
        names{j} = flist(i).name;
        I{j} = imread(sprintf('%s/%s',D,flist(i).name));  
        try %if I{j} is not binary
            image_bw = ~im2bw(I{j},graythresh(I{j}));
        catch
            image_bw = ~I{j};
        end
        %reduce the printing noise
        images(:,:,j) = normalize_img(image_bw, SizeX, SizeY);
    end
    i = i + 1;
end
N = j;
end

%%
function img_nor = normalize_img(img,SizeX, SizeY)
    img_nor = normalize_img1(img,SizeX,SizeY);
end


%% Normalization from Qiang
function img_nor = normalize_img1(image_bw,SizeX,SizeY)    
%     image_d = bwmorph(image_bw,'dilate');
%     image_m = bwmorph(image_d,'majority');
%     image_thin = bwmorph(image_m,'thin',3);%Thinning
%     image_bw = bwmorph(image_thin,'remove'); %Removes interior pixels
    
    img = ~image_bw;
    %normalize img to SIZE*SIZE
    [idy idx] = find(img == 0);
    minx = min(idx);
    maxx = max(idx);
    miny = min(idy);
    maxy = max(idy);
    xm = (minx+maxx)/2;
    ym = (miny+maxy)/2;
    scale = max((maxx-minx)/2,(maxy-miny)/2);

    img_nor = ones(SizeX,SizeY);
    for i = 1:length(idy)
        y = round(SizeX/2+SizeX/2*(idy(i)-ym)/scale);
        x = round(SizeY/2+SizeY/2*(idx(i)-xm)/scale);
        if y == 0
            y = 1;
        end
        if x == 0
            x = 1;
        end
        img_nor(y,x) = 0;
    end    
end

